@@ -17,7 +17,7 @@
 # under the License.
 #
 
-our $VERSION = '0.9.2';
+our $VERSION = '0.9.3';
 
 require 5.6.0;
 use strict;
@@ -84,7 +84,7 @@ use constant UNSUPPORTED_CLIENT_TYPE => 10;
 sub new {
     my $classname = shift;
 
-    my $self = $classname->SUPER::new();
+    my $self = $classname->SUPER::new(@_);
 
     return bless($self,$classname);
 }
