@@ -29,9 +29,6 @@ use Thrift::BinaryProtocol;
 # Server base class module
 #
 package Thrift::Server;
-{
-  $Thrift::Server::VERSION = '0.9.0';
-}
 
 # 3 possible constructors:
 #   1.  (processor, serverTransport)
@@ -118,8 +115,8 @@ sub _handleException
         my $out     = $code . ':' . $message;
 
         $message =~ m/TTransportException/ and die $out;
-        if ($message =~ m/TSocket/) {
-            # suppress TSocket messages
+        if ($message =~ m/Socket/) {
+            # suppress Socket messages
         } else {
             warn $out;
         }
@@ -133,9 +130,6 @@ sub _handleException
 # SimpleServer from the Server base class that handles one connection at a time
 #
 package Thrift::SimpleServer;
-{
-  $Thrift::SimpleServer::VERSION = '0.9.0';
-}
 use base qw( Thrift::Server );
 
 sub new
@@ -179,9 +173,6 @@ sub serve
 # ForkingServer that forks a new process for each request
 #
 package Thrift::ForkingServer;
-{
-  $Thrift::ForkingServer::VERSION = '0.9.0';
-}
 use base qw( Thrift::Server );
 
 use POSIX ":sys_wait_h";
